@@ -7,40 +7,31 @@ const PERMISSIONS_PAGE =
     "chrome://mochitests/content",
     "https://example.com"
   ) + "permissions.html";
-const kStrictKeyPressEvents = SpecialPowers.getBoolPref(
-  "dom.keyboardevent.keypress.dispatch_non_printable_keys_only_system_group_in_content"
-);
 
-function openIdentityPopup() {
-  let promise = BrowserTestUtils.waitForEvent(
-    gIdentityHandler._identityPopup,
-    "popupshown"
+function testPermListHasEntries(expectEntries) {
+  let permissionsList = document.getElementById(
+    "permission-popup-permission-list"
   );
-  gIdentityHandler._identityBox.click();
-  return promise;
-}
-
-function closeIdentityPopup() {
-  let promise = BrowserTestUtils.waitForEvent(
-    gIdentityHandler._identityPopup,
-    "popuphidden"
-  );
-  gIdentityHandler._identityPopup.hidePopup();
-  return promise;
+  let listEntryCount = permissionsList.querySelectorAll(
+    ".permission-popup-permission-item"
+  ).length;
+  if (expectEntries) {
+    ok(listEntryCount, "List of permissions is not empty");
+    return;
+  }
+  ok(!listEntryCount, "List of permissions is empty");
 }
 
 add_task(async function testMainViewVisible() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function() {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function () {
+    await openPermissionPopup();
+
     let permissionsList = document.getElementById(
-      "identity-popup-permission-list"
+      "permission-popup-permission-list"
     );
-    let emptyLabel = permissionsList.nextElementSibling.nextElementSibling;
+    testPermListHasEntries(false);
 
-    await openIdentityPopup();
-
-    ok(!BrowserTestUtils.is_hidden(emptyLabel), "List of permissions is empty");
-
-    await closeIdentityPopup();
+    await closePermissionPopup();
 
     PermissionTestUtils.add(
       gBrowser.currentURI,
@@ -48,40 +39,37 @@ add_task(async function testMainViewVisible() {
       Services.perms.ALLOW_ACTION
     );
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
-    ok(
-      BrowserTestUtils.is_hidden(emptyLabel),
-      "List of permissions is not empty"
-    );
+    testPermListHasEntries(true);
 
     let labelText = SitePermissions.getPermissionLabel("camera");
     let labels = permissionsList.querySelectorAll(
-      ".identity-popup-permission-label"
+      ".permission-popup-permission-label"
     );
     is(labels.length, 1, "One permission visible in main view");
-    is(labels[0].textContent, labelText, "Correct value");
+    is(labels[0].innerHTML, labelText, "Correct value");
 
     let img = permissionsList.querySelector(
-      "image.identity-popup-permission-icon"
+      "image.permission-popup-permission-icon"
     );
     ok(img, "There is an image for the permissions");
     ok(img.classList.contains("camera-icon"), "proper class is in image class");
 
-    await closeIdentityPopup();
+    await closePermissionPopup();
 
     PermissionTestUtils.remove(gBrowser.currentURI, "camera");
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
-    ok(!BrowserTestUtils.is_hidden(emptyLabel), "List of permissions is empty");
+    testPermListHasEntries(false);
 
-    await closeIdentityPopup();
+    await closePermissionPopup();
   });
 });
 
 add_task(async function testIdentityIcon() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, function() {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, function () {
     PermissionTestUtils.add(
       gBrowser.currentURI,
       "geo",
@@ -89,25 +77,25 @@ add_task(async function testIdentityIcon() {
     );
 
     ok(
-      gIdentityHandler._identityBox.classList.contains("grantedPermissions"),
+      gPermissionPanel._identityPermissionBox.hasAttribute("hasPermissions"),
       "identity-box signals granted permissions"
     );
 
     PermissionTestUtils.remove(gBrowser.currentURI, "geo");
 
     ok(
-      !gIdentityHandler._identityBox.classList.contains("grantedPermissions"),
+      !gPermissionPanel._identityPermissionBox.hasAttribute("hasPermissions"),
       "identity-box doesn't signal granted permissions"
     );
 
     PermissionTestUtils.add(
       gBrowser.currentURI,
-      "camera",
-      Services.perms.DENY_ACTION
+      "not-a-site-permission",
+      Services.perms.ALLOW_ACTION
     );
 
     ok(
-      !gIdentityHandler._identityBox.classList.contains("grantedPermissions"),
+      !gPermissionPanel._identityPermissionBox.hasAttribute("hasPermissions"),
       "identity-box doesn't signal granted permissions"
     );
 
@@ -118,22 +106,45 @@ add_task(async function testIdentityIcon() {
     );
 
     ok(
-      gIdentityHandler._identityBox.classList.contains("grantedPermissions"),
+      gPermissionPanel._identityPermissionBox.hasAttribute("hasPermissions"),
       "identity-box signals granted permissions"
     );
 
+    PermissionTestUtils.remove(gBrowser.currentURI, "cookie");
+
+    PermissionTestUtils.add(
+      gBrowser.currentURI,
+      "cookie",
+      Ci.nsICookiePermission.ACCESS_DENY
+    );
+
+    ok(
+      gPermissionPanel._identityPermissionBox.hasAttribute("hasPermissions"),
+      "identity-box signals granted permissions"
+    );
+
+    PermissionTestUtils.add(
+      gBrowser.currentURI,
+      "cookie",
+      Ci.nsICookiePermission.ACCESS_DEFAULT
+    );
+
+    ok(
+      !gPermissionPanel._identityPermissionBox.hasAttribute("hasPermissions"),
+      "identity-box doesn't signal granted permissions"
+    );
+
     PermissionTestUtils.remove(gBrowser.currentURI, "geo");
-    PermissionTestUtils.remove(gBrowser.currentURI, "camera");
+    PermissionTestUtils.remove(gBrowser.currentURI, "not-a-site-permission");
     PermissionTestUtils.remove(gBrowser.currentURI, "cookie");
   });
 });
 
 add_task(async function testCancelPermission() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function() {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function () {
     let permissionsList = document.getElementById(
-      "identity-popup-permission-list"
+      "permission-popup-permission-list"
     );
-    let emptyLabel = permissionsList.nextElementSibling.nextElementSibling;
 
     PermissionTestUtils.add(
       gBrowser.currentURI,
@@ -146,57 +157,50 @@ add_task(async function testCancelPermission() {
       Services.perms.DENY_ACTION
     );
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
-    ok(
-      BrowserTestUtils.is_hidden(emptyLabel),
-      "List of permissions is not empty"
-    );
+    testPermListHasEntries(true);
 
     permissionsList
-      .querySelector(".identity-popup-permission-remove-button")
+      .querySelector(".permission-popup-permission-remove-button")
       .click();
 
     is(
-      permissionsList.querySelectorAll(".identity-popup-permission-label")
+      permissionsList.querySelectorAll(".permission-popup-permission-label")
         .length,
       1,
       "First permission should be removed"
     );
 
     permissionsList
-      .querySelector(".identity-popup-permission-remove-button")
+      .querySelector(".permission-popup-permission-remove-button")
       .click();
 
     is(
-      permissionsList.querySelectorAll(".identity-popup-permission-label")
+      permissionsList.querySelectorAll(".permission-popup-permission-label")
         .length,
       0,
       "Second permission should be removed"
     );
 
-    await closeIdentityPopup();
+    await closePermissionPopup();
   });
 });
 
 add_task(async function testPermissionHints() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function(browser) {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function (browser) {
     let permissionsList = document.getElementById(
-      "identity-popup-permission-list"
-    );
-    let emptyHint = document.getElementById(
-      "identity-popup-permission-empty-hint"
+      "permission-popup-permission-list"
     );
     let reloadHint = document.getElementById(
-      "identity-popup-permission-reload-hint"
+      "permission-popup-permission-reload-hint"
     );
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
-    ok(!BrowserTestUtils.is_hidden(emptyHint), "Empty hint is visible");
     ok(BrowserTestUtils.is_hidden(reloadHint), "Reload hint is hidden");
 
-    await closeIdentityPopup();
+    await closePermissionPopup();
 
     PermissionTestUtils.add(
       gBrowser.currentURI,
@@ -209,45 +213,38 @@ add_task(async function testPermissionHints() {
       Services.perms.DENY_ACTION
     );
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
-    ok(BrowserTestUtils.is_hidden(emptyHint), "Empty hint is hidden");
     ok(BrowserTestUtils.is_hidden(reloadHint), "Reload hint is hidden");
 
     let cancelButtons = permissionsList.querySelectorAll(
-      ".identity-popup-permission-remove-button"
+      ".permission-popup-permission-remove-button"
     );
     PermissionTestUtils.remove(gBrowser.currentURI, "camera");
 
     cancelButtons[0].click();
-    ok(BrowserTestUtils.is_hidden(emptyHint), "Empty hint is hidden");
     ok(!BrowserTestUtils.is_hidden(reloadHint), "Reload hint is visible");
 
     cancelButtons[1].click();
-    ok(BrowserTestUtils.is_hidden(emptyHint), "Empty hint is hidden");
     ok(!BrowserTestUtils.is_hidden(reloadHint), "Reload hint is visible");
 
-    await closeIdentityPopup();
+    await closePermissionPopup();
     let loaded = BrowserTestUtils.browserLoaded(browser);
-    BrowserTestUtils.loadURI(browser, PERMISSIONS_PAGE);
+    BrowserTestUtils.loadURIString(browser, PERMISSIONS_PAGE);
     await loaded;
-    await openIdentityPopup();
+    await openPermissionPopup();
 
-    ok(
-      !BrowserTestUtils.is_hidden(emptyHint),
-      "Empty hint is visible after reloading"
-    );
     ok(
       BrowserTestUtils.is_hidden(reloadHint),
       "Reload hint is hidden after reloading"
     );
 
-    await closeIdentityPopup();
+    await closePermissionPopup();
   });
 });
 
 add_task(async function testPermissionIcons() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, function() {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, function () {
     PermissionTestUtils.add(
       gBrowser.currentURI,
       "camera",
@@ -259,12 +256,12 @@ add_task(async function testPermissionIcons() {
       Services.perms.DENY_ACTION
     );
 
-    let geoIcon = gIdentityHandler._identityBox.querySelector(
+    let geoIcon = gPermissionPanel._identityPermissionBox.querySelector(
       ".blocked-permission-icon[data-permission-id='geo']"
     );
     ok(geoIcon.hasAttribute("showing"), "blocked permission icon is shown");
 
-    let cameraIcon = gIdentityHandler._identityBox.querySelector(
+    let cameraIcon = gPermissionPanel._identityPermissionBox.querySelector(
       ".blocked-permission-icon[data-permission-id='camera']"
     );
     ok(
@@ -284,7 +281,7 @@ add_task(async function testPermissionIcons() {
 });
 
 add_task(async function testPermissionShortcuts() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function(browser) {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function (browser) {
     browser.focus();
 
     await new Promise(r => {
@@ -296,7 +293,7 @@ add_task(async function testPermissionShortcuts() {
 
     async function tryKey(desc, expectedValue) {
       await EventUtils.synthesizeAndWaitKey("c", { accelKey: true });
-      let result = await SpecialPowers.spawn(browser, [], function() {
+      let result = await SpecialPowers.spawn(browser, [], function () {
         return {
           keydowns: content.wrappedJSObject.gKeyDowns,
           keypresses: content.wrappedJSObject.gKeyPresses,
@@ -307,19 +304,11 @@ add_task(async function testPermissionShortcuts() {
         expectedValue,
         "keydown event was fired or not fired as expected, " + desc
       );
-      if (kStrictKeyPressEvents) {
-        is(
-          result.keypresses,
-          0,
-          "keypress event shouldn't be fired for shortcut key, " + desc
-        );
-      } else {
-        is(
-          result.keypresses,
-          expectedValue,
-          "keypress event should be fired even for shortcut key, " + desc
-        );
-      }
+      is(
+        result.keypresses,
+        0,
+        "keypress event shouldn't be fired for shortcut key, " + desc
+      );
     }
 
     await tryKey("pressed with default permissions", 1);
@@ -367,13 +356,13 @@ add_task(async function testPermissionShortcuts() {
 
 // Test the control center UI when policy permissions are set.
 add_task(async function testPolicyPermission() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function() {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function () {
     await SpecialPowers.pushPrefEnv({
       set: [["dom.disable_open_during_load", true]],
     });
 
     let permissionsList = document.getElementById(
-      "identity-popup-permission-list"
+      "permission-popup-permission-list"
     );
     PermissionTestUtils.add(
       gBrowser.currentURI,
@@ -382,57 +371,59 @@ add_task(async function testPolicyPermission() {
       Services.perms.EXPIRE_POLICY
     );
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
     // Check if the icon, nameLabel and stateLabel are visible.
     let img, labelText, labels;
 
-    img = permissionsList.querySelector("image.identity-popup-permission-icon");
+    img = permissionsList.querySelector(
+      "image.permission-popup-permission-icon"
+    );
     ok(img, "There is an image for the popup permission");
     ok(img.classList.contains("popup-icon"), "proper class is in image class");
 
     labelText = SitePermissions.getPermissionLabel("popup");
     labels = permissionsList.querySelectorAll(
-      ".identity-popup-permission-label"
+      ".permission-popup-permission-label"
     );
     is(labels.length, 1, "One permission visible in main view");
-    is(labels[0].textContent, labelText, "Correct name label value");
+    is(labels[0].innerHTML, labelText, "Correct name label value");
 
     labelText = SitePermissions.getCurrentStateLabel(
       SitePermissions.ALLOW,
       SitePermissions.SCOPE_POLICY
     );
     labels = permissionsList.querySelectorAll(
-      ".identity-popup-permission-state-label"
+      ".permission-popup-permission-state-label"
     );
-    is(labels[0].textContent, labelText, "Correct state label value");
+    is(labels[0].innerHTML, labelText, "Correct state label value");
 
     // Check if the menulist and the remove button are hidden.
     // The menulist is specific to the "popup" permission.
-    let menulist = document.getElementById("identity-popup-popup-menulist");
+    let menulist = document.getElementById("permission-popup-menulist");
     ok(menulist == null, "The popup permission menulist is not visible");
 
     let removeButton = permissionsList.querySelector(
-      ".identity-popup-permission-remove-button"
+      ".permission-popup-permission-remove-button"
     );
     ok(removeButton == null, "The permission remove button is not visible");
 
     Services.perms.removeAll();
-    await closeIdentityPopup();
+    await closePermissionPopup();
   });
 });
 
 add_task(async function testHiddenAfterRefresh() {
-  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function(browser) {
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function (browser) {
     ok(
-      BrowserTestUtils.is_hidden(gIdentityHandler._identityPopup),
+      BrowserTestUtils.is_hidden(gPermissionPanel._permissionPopup),
       "Popup is hidden"
     );
 
-    await openIdentityPopup();
+    await openPermissionPopup();
 
     ok(
-      !BrowserTestUtils.is_hidden(gIdentityHandler._identityPopup),
+      !BrowserTestUtils.is_hidden(gPermissionPanel._permissionPopup),
       "Popup is shown"
     );
 
@@ -445,8 +436,134 @@ add_task(async function testHiddenAfterRefresh() {
     await reloaded;
 
     ok(
-      BrowserTestUtils.is_hidden(gIdentityHandler._identityPopup),
+      BrowserTestUtils.is_hidden(gPermissionPanel._permissionPopup),
       "Popup is hidden"
     );
   });
 });
+
+add_task(async function test3rdPartyStoragePermission() {
+  // 3rdPartyStorage permissions are listed under an anchor container - test
+  // that this works correctly, i.e. the permission items are added to the
+  // anchor when relevant, and other permission items are added to the default
+  // anchor, and adding/removing permissions preserves this behavior correctly.
+
+  await BrowserTestUtils.withNewTab(PERMISSIONS_PAGE, async function (browser) {
+    await openPermissionPopup();
+
+    let permissionsList = document.getElementById(
+      "permission-popup-permission-list"
+    );
+    let storagePermissionAnchor = permissionsList.querySelector(
+      `.permission-popup-permission-list-anchor[anchorfor="3rdPartyStorage"]`
+    );
+
+    testPermListHasEntries(false);
+
+    ok(
+      BrowserTestUtils.is_hidden(storagePermissionAnchor.firstElementChild),
+      "Anchor header is hidden"
+    );
+
+    await closePermissionPopup();
+
+    let storagePermissionID = "3rdPartyStorage^example2.com";
+    PermissionTestUtils.add(
+      browser.currentURI,
+      storagePermissionID,
+      Services.perms.ALLOW_ACTION
+    );
+
+    await openPermissionPopup();
+
+    testPermListHasEntries(true);
+    ok(
+      BrowserTestUtils.is_visible(storagePermissionAnchor.firstElementChild),
+      "Anchor header is visible"
+    );
+
+    let labelText = SitePermissions.getPermissionLabel(storagePermissionID);
+    let labels = storagePermissionAnchor.querySelectorAll(
+      ".permission-popup-permission-label"
+    );
+    is(labels.length, 1, "One permission visible in 3rdPartyStorage anchor");
+    is(
+      labels[0].getAttribute("value"),
+      labelText,
+      "Permission label has the correct value"
+    );
+
+    await closePermissionPopup();
+
+    PermissionTestUtils.add(
+      browser.currentURI,
+      "camera",
+      Services.perms.ALLOW_ACTION
+    );
+
+    await openPermissionPopup();
+
+    testPermListHasEntries(true);
+    ok(
+      BrowserTestUtils.is_visible(storagePermissionAnchor.firstElementChild),
+      "Anchor header is visible"
+    );
+
+    labels = permissionsList.querySelectorAll(
+      ".permission-popup-permission-label"
+    );
+    is(labels.length, 2, "Two permissions visible in main view");
+    labels = storagePermissionAnchor.querySelectorAll(
+      ".permission-popup-permission-label"
+    );
+    is(labels.length, 1, "One permission visible in 3rdPartyStorage anchor");
+
+    storagePermissionAnchor
+      .querySelector(".permission-popup-permission-remove-button")
+      .click();
+    is(
+      storagePermissionAnchor.querySelectorAll(
+        ".permission-popup-permission-label"
+      ).length,
+      0,
+      "Permission item should be removed"
+    );
+    is(
+      PermissionTestUtils.testPermission(
+        browser.currentURI,
+        storagePermissionID
+      ),
+      SitePermissions.UNKNOWN,
+      "Permission removed from permission manager"
+    );
+
+    await closePermissionPopup();
+
+    await openPermissionPopup();
+
+    testPermListHasEntries(true);
+    ok(
+      BrowserTestUtils.is_hidden(storagePermissionAnchor.firstElementChild),
+      "Anchor header is hidden"
+    );
+
+    labels = permissionsList.querySelectorAll(
+      ".permission-popup-permission-label"
+    );
+    is(labels.length, 1, "One permission visible in main view");
+
+    await closePermissionPopup();
+
+    PermissionTestUtils.remove(browser.currentURI, "camera");
+
+    await openPermissionPopup();
+
+    testPermListHasEntries(false);
+    ok(
+      BrowserTestUtils.is_hidden(storagePermissionAnchor.firstElementChild),
+      "Anchor header is hidden"
+    );
+
+    await closePermissionPopup();
+  });
+});
