@@ -1,11 +1,11 @@
-const { PanelTestProvider } = ChromeUtils.import(
-  "resource://activity-stream/lib/PanelTestProvider.jsm"
+const { PanelTestProvider } = ChromeUtils.importESModule(
+  "resource://activity-stream/lib/PanelTestProvider.sys.mjs"
 );
 const { MomentsPageHub } = ChromeUtils.import(
   "resource://activity-stream/lib/MomentsPageHub.jsm"
 );
-const { RemoteSettings } = ChromeUtils.import(
-  "resource://services-settings/remote-settings.js"
+const { RemoteSettings } = ChromeUtils.importESModule(
+  "resource://services-settings/remote-settings.sys.mjs"
 );
 const { ASRouter } = ChromeUtils.import(
   "resource://activity-stream/lib/ASRouter.jsm"
@@ -19,7 +19,7 @@ add_task(async function test_with_rs_messages() {
     set: [
       [
         "browser.newtabpage.activity-stream.asrouter.providers.whats-new-panel",
-        `{"id":"cfr","enabled":true,"type":"remote-settings","bucket":"cfr","updateCycleInMs":0}`,
+        `{"id":"cfr","enabled":true,"type":"remote-settings","collection":"cfr","updateCycleInMs":0}`,
       ],
     ],
   });
@@ -44,11 +44,12 @@ add_task(async function test_with_rs_messages() {
   );
   // Reload the provider
   await ASRouter._updateMessageProviders();
+  await ASRouter.loadMessagesFromAllProviders();
   // Wait to load the WNPanel messages
-  await BrowserTestUtils.waitForCondition(async () => {
-    await ASRouter.loadMessagesFromAllProviders();
-    return ASRouter.state.messages.length > initialMessageCount;
-  }, "Messages did not load");
+  await BrowserTestUtils.waitForCondition(
+    () => ASRouter.state.messages.length > initialMessageCount,
+    "Messages did not load"
+  );
 
   await MomentsPageHub.messageRequest({
     triggerId: "momentsUpdate",
@@ -79,10 +80,11 @@ add_task(async function test_with_rs_messages() {
 
   let prevLength = ASRouter.state.messages.length;
   // Wait to load the messages
-  await BrowserTestUtils.waitForCondition(async () => {
-    await ASRouter.loadMessagesFromAllProviders();
-    return ASRouter.state.messages.length > prevLength;
-  }, "Messages did not load");
+  await ASRouter.loadMessagesFromAllProviders();
+  await BrowserTestUtils.waitForCondition(
+    () => ASRouter.state.messages.length > prevLength,
+    "Messages did not load"
+  );
 
   await MomentsPageHub.messageRequest({
     triggerId: "momentsUpdate",
@@ -103,10 +105,11 @@ add_task(async function test_with_rs_messages() {
   await client.db.clear();
   // Wait to reset the WNPanel messages from state
   const previousMessageCount = ASRouter.state.messages.length;
-  await BrowserTestUtils.waitForCondition(async () => {
-    await ASRouter.loadMessagesFromAllProviders();
-    return ASRouter.state.messages.length < previousMessageCount;
-  }, "ASRouter messages should have been removed");
+  await ASRouter.loadMessagesFromAllProviders();
+  await BrowserTestUtils.waitForCondition(
+    () => ASRouter.state.messages.length < previousMessageCount,
+    "ASRouter messages should have been removed"
+  );
   await SpecialPowers.popPrefEnv();
   // Reload the provider
   await ASRouter._updateMessageProviders();
